package org.moviepedia;

import java.io.File;
import java.io.FileNotFoundException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Iterator;

import org.moviepedia.moviedb.DbFactory;
import org.moviepedia.moviedb.Movie;
import org.moviepedia.moviedb.MovieDb;
import org.moviepedia.moviedb.MovieSearch;

import android.app.SearchManager;
import android.content.ContentProvider;
import android.content.ContentValues;
import android.content.UriMatcher;
import android.content.res.AssetFileDescriptor;
import android.database.Cursor;
import android.database.MatrixCursor;
import android.net.Uri;
import android.os.ParcelFileDescriptor;
import android.provider.BaseColumns;

public class MovieProvider extends ContentProvider
{

	private static final String AUTHORITY = "org.moviepedia.provider";

	/***************************************** Contracts *****************************************/
	public static class MovieContract
	{
		public static String _ID = BaseColumns._ID;
		public static String _TITLE = "title";
		public static String _RELEASE_DATE = "release_date";

		public static String[] COLUMNS = new String[] {_ID, _TITLE, _RELEASE_DATE};
		
		public static String CONTENT_ITEM_TYPE = "vnd.android.cursor.item/vnd.org.moviepedia.provider.movie";
		public static String CONTENT_TYPE = "vnd.android.cursor.dir/vnd.org.moviepedia.provider.movie";
	}
	
	public static class SuggestionContract
	{
		public static String _SELECTION = "query";
		public static String _ID = BaseColumns._ID;
		public static String _TITLE = SearchManager.SUGGEST_COLUMN_TEXT_1;
//		public static String _RELEASE_DATE = SearchManager.SUGGEST_COLUMN_TEXT_2;
		public static String _ICON = SearchManager.SUGGEST_COLUMN_ICON_1;
		
		public static String[] COLUMNS = new String[] { _ID, _TITLE, _ICON };
	}

	/***************************************** Uri processing *****************************************/
	private static final int MOVIE = 1;
	private static final int MOVIE_ID = 2;
	private static final int SUGGESTION = 3;
	private static final int RESOURCE_ID = 4;
	
	private static final UriMatcher URI_MATCHER = new UriMatcher(UriMatcher.NO_MATCH);
	static
	{
		URI_MATCHER.addURI(AUTHORITY, "movie", MOVIE);
		URI_MATCHER.addURI(AUTHORITY, "movie/#", MOVIE_ID);
		URI_MATCHER.addURI(AUTHORITY, "search_suggest_query", SUGGESTION);
		URI_MATCHER.addURI(AUTHORITY, "resource/*", RESOURCE_ID);
	}
	
	@Override
	public String getType(Uri uri)
	{
		int match = URI_MATCHER.match(uri);
		switch (match)
		{
			case MOVIE:
				return MovieContract.CONTENT_TYPE;
			case MOVIE_ID:
				return MovieContract.CONTENT_ITEM_TYPE;
			case SUGGESTION:
				return MovieContract.CONTENT_TYPE;
			default:
				return null;
		}
	}
	
	/***************************************** Initialization *****************************************/
	
	@Override
	public boolean onCreate()
	{
		return true;
	}
	
	/***************************************** Querying *****************************************/
	
	@Override
	public Cursor query(Uri uri, String[] projection, String selection,
			String[] selectionArgs, String sortOrder)
	{

		int match = URI_MATCHER.match(uri);
		switch (match)
		{
			case MOVIE:
				return queryMovies(uri, projection, selection,
						selectionArgs, sortOrder, Integer.MAX_VALUE);
			case MOVIE_ID:
				return queryMovie(uri, projection, selection,
						selectionArgs, sortOrder);
			case SUGGESTION:
				return searchSuggestions(uri, projection, selection,
						selectionArgs, sortOrder);
		}
		return null;
	}

	private Cursor searchSuggestions(Uri uri, String[] projection,
			String selection, String[] selectionArgs, String sortOrder)
	{
		if ( !SuggestionContract._SELECTION.equals(selection) )
		{
			throw new IllegalArgumentException(String.format(
					"Selection=%s not valid. Expected selection=%s", selection,
					SuggestionContract._SELECTION));
		}
		if ( selectionArgs == null || selectionArgs.length != 1 )
		{
			throw new IllegalArgumentException("Expected selectionArgs.length==1");
		}
		MatrixCursor cursor = new MatrixCursor( SuggestionContract.COLUMNS );
		String query = selectionArgs[0];
		if ( query.length() > 0 )
		{
			MovieSearch search = searchMovies(query);
	        Iterator<Movie> iterator = search.iterator();
	        for ( int i=0 ; i<4 && iterator.hasNext() ; i++ )
	        {
	        	Movie movie = iterator.next();
	        	Object[] movieElems = new Object[ SuggestionContract.COLUMNS.length ];
	        	movieElems[0] = movie.getId();
	        	movieElems[1] = movie.getTitle();
	        	String posterUrl = movie.getPosterUrl();
	        	if ( posterUrl != null )
	        	{
	        		movieElems[2] = "content://"+AUTHORITY+"/resource/" + Uri.encode(posterUrl);//movie.getReleaseDate();
	        	}
	        	else
	        	{
	        		movieElems[2] = null;
	        	}
	        	cursor.addRow(movieElems);
	        }
		}
		return cursor;
	}
	
	private ParcelFileDescriptor getFileDescriptor(Uri uri) throws FileNotFoundException
	{
		String url = Uri.decode( uri.getLastPathSegment() );
		
		try
		{
			URL myUrl = new URL(url);
			ParcelFileDescriptor descr = ParcelFileDescriptor.open( new File(myUrl.getFile()), ParcelFileDescriptor.MODE_READ_ONLY);
			return descr;
		} catch (MalformedURLException e)
		{
			throw new FileNotFoundException(e.getMessage());
		}
	}

	@Override
	public ParcelFileDescriptor openFile(Uri uri, String mode)
			throws FileNotFoundException {
		
		int match = URI_MATCHER.match(uri);
		switch (match)
		{
			case RESOURCE_ID:
				return getFileDescriptor(uri);
		}
		
		ParcelFileDescriptor file = super.openFile(uri, mode);
		return file;
	}
	
	private Cursor queryMovie(Uri uri, String[] projection, String selection,
			String[] selectionArgs, String sortOrder)
	{
		// TODO Auto-generated method stub
		return null;
	}

	private Cursor queryMovies(Uri uri, String[] projection, String selection,
			String[] selectionArgs, String sortOrder, int limit)
	{
		MatrixCursor cursor = new MatrixCursor( MovieContract.COLUMNS );
		String query = uri.getLastPathSegment();
		MovieSearch search = searchMovies(query);
        Iterator<Movie> iterator = search.iterator();
        for ( int i=0 ; i<limit && iterator.hasNext() ; i++ )
        {
        	Movie movie = iterator.next();
        	Object[] movieElems = new Object[3];
        	movieElems[0] = movie.getId();
        	movieElems[1] = movie.getTitle();
        	movieElems[2] = movie.getReleaseDate();
        	cursor.addRow(movieElems);
        }
		return cursor;
	}

	private MovieSearch searchMovies(String query)
	{
		MovieDb movieDb = DbFactory.get();
        movieDb.connect();
        MovieSearch search = movieDb.search(query);
        return search;
	}

	/***************************************** Read only *****************************************/
	
	@Override
	public int delete(Uri uri, String selection, String[] selectionArgs)
	{
		return 0;
	}

	@Override
	public int update(Uri uri, ContentValues values, String selection,
			String[] selectionArgs)
	{
		return 0;
	}

	@Override
	public Uri insert(Uri uri, ContentValues values)
	{
		return null;
	}

}
